#ifndef JSONEDIT_JSONCONS_H
#define JSONEDIT_JSONCONS_H

#	define JE_JSONCONS_CALL_CONV __cdecl

/** \def DECLARE_FN
	\brief Declare function or consistent function + function pointer set depending on JSONEDIT_JSONCONS_EXPORTS
*/
#ifdef JSONEDIT_JSONCONS_EXPORTS
#define DECLARE_FN(type, fn, ...) __declspec(dllexport) type JE_JSONCONS_CALL_CONV fn(__VA_ARGS__)
#else
#define DECLARE_FN(type, fn, ...) __declspec(dllimport) type JE_JSONCONS_CALL_CONV fn(__VA_ARGS__); \
	typedef type (JE_JSONCONS_CALL_CONV *pf##fn)(__VA_ARGS__)
#endif


#ifdef __cplusplus
extern "C" {
#endif


	/** \brief Dll interface version, used to dismiss outdated libraries
	*/
	enum { JE_JSONCONS_DLL_INTERFACE_MAJOR_VERSION = 1 };
	/** \brief Dll interface version; used to dismiss outdated libraries
	*/
	enum { JE_JSONCONS_DLL_INTERFACE_MINOR_VERSION = 0 };

	/** \brief Holds Dll interface version
	*/
	struct JE_JSONCONS_DLL_INTERFACE
	{
		int majorVersion;   ///< major part of interface version
		int minorVersion;   ///< minor part of interface version
	};

	/** \brief Get dll interface version to check for compatibility
	*/
	DECLARE_FN(void, JE_jsoncons_get_interface_version, struct JE_JSONCONS_DLL_INTERFACE*);

	DECLARE_FN(int, JE_jsoncons_jsonpath_query, const char* jsonText, const char* path, const char** ret, const char** exception_text);

	DECLARE_FN(int, JE_jsoncons_jmespath_search, const char* jsonText, const char* expr, const char** ret, const char** exception_text);

	DECLARE_FN(int, JE_jsoncons_merge_patch_apply, const char* jsonText, const char* jsonPatch, const char** ret, const char** exception_text);

	DECLARE_FN(int, JE_jsoncons_merge_patch_create, const char* jsonSource, const char* jsonTarget, const char** ret, const char** exception_text);

	DECLARE_FN(int, JE_jsoncons_patch_apply, const char* jsonText, const char* jsonPatch, const char** ret, const char** exception_text);

	DECLARE_FN(int, JE_jsoncons_patch_create, const char* jsonSource, const char* jsonTarget, const char** ret, const char** exception_text);

	DECLARE_FN(void, JE_jsoncons_release, void);

#ifdef __cplusplus
}
#endif

#endif