#include <iostream>
#include <stdio.h>
#include <windows.h>
#include "../JSONedit_jsoncons/JSONedit_jsoncons.h"

int main()
{
    std::cout << "JsonEdit jsoncons DLL test\r\n";
	HINSTANCE hInstance = LoadLibrary(L"JSONedit_jsoncons.dll");
	if (!hInstance)
	{
		printf("Failed to load DLL!\r\n");
		return -1;
	}

	pfJE_jsoncons_get_interface_version dllJE_jsoncons_get_interface_version = (pfJE_jsoncons_get_interface_version)GetProcAddress(hInstance, "JE_jsoncons_get_interface_version");
	if (!dllJE_jsoncons_get_interface_version)
	{
		printf("Function JE_jsoncons_get_interface_version not found!\r\n");
		FreeLibrary(hInstance);
		return -2;
	}
	struct JE_JSONCONS_DLL_INTERFACE version;
	dllJE_jsoncons_get_interface_version(&version);
	if (version.majorVersion != JE_JSONCONS_DLL_INTERFACE_MAJOR_VERSION || version.minorVersion != JE_JSONCONS_DLL_INTERFACE_MINOR_VERSION)
	{
		printf("Invalid DLL interface version!\r\n");
		FreeLibrary(hInstance);
		return -2;
	}

	pfJE_jsoncons_jsonpath_query dllJE_jsoncons_jsonpath_query = (pfJE_jsoncons_jsonpath_query)GetProcAddress(hInstance, "JE_jsoncons_jsonpath_query");

	if (dllJE_jsoncons_jsonpath_query)
	{
		const char* json =
			"{\n"
			"   \"menu\" : {\n"
			"      \"id\" : \"file\",\n"
			"      \"popup\" : {\n"
			"         \"menuitem\" : [\n"
			"            {\n"
			"               \"onclick\" : \"CreateNewDoc()\",\n"
			"               \"value\" : \"New\"\n"
			"            },\n"
			"            {\n"
			"               \"onclick\" : \"OpenDoc()\",\n"
			"               \"value\" : \"Open\"\n"
			"            },\n"
			"            {\n"
			"               \"onclick\" : \"CloseDoc()\",\n"
			"               \"value\" : \"Close\"\n"
			"            }\n"
			"         ]\n"
			"      },\n"
			"      \"value\" : \"File\"\n"
			"   }\n"
			"}\n"
			"";
		const char* path = "$.menu.popup";
		const char* ret = "";
		const char* exceptionText = "";
		int status = dllJE_jsoncons_jsonpath_query(json, path, &ret, &exceptionText);
		printf("JSONPath return: %d [EXC: %s], %s\n", status, exceptionText, ret);
	}

	pfJE_jsoncons_jmespath_search dllJE_jsoncons_jmespath_search = (pfJE_jsoncons_jmespath_search)GetProcAddress(hInstance, "JE_jsoncons_jmespath_search");
	if (dllJE_jsoncons_jmespath_search)
	{
		std::string jtext = R"(
		{
		  "locations": [
			{"name": "Seattle", "state": "WA"},
			{"name": "New York", "state": "NY"},
			{"name": "Bellevue", "state": "WA"},
			{"name": "Olympia", "state": "WA"}
		  ]
		}        
		)";
		
		std::string expr = "locations[?state == 'WA'].name | sort(@) | {WashingtonCities: join(', ', @)}";
		const char* ret = "";
		const char* exceptionText = "";

		int status = dllJE_jsoncons_jmespath_search(jtext.c_str(), expr.c_str(), &ret, &exceptionText);
		printf("JMESPath return: %d [EXC: %s], %s\n", status, exceptionText, ret);
	}

	{
		pfJE_jsoncons_merge_patch_apply dllJE_jsoncons_merge_patch_apply = (pfJE_jsoncons_merge_patch_apply)GetProcAddress(hInstance, "JE_jsoncons_merge_patch_apply");
		if (dllJE_jsoncons_merge_patch_apply)
		{
			std::string jtext = R"(
						{
								 "title": "Goodbye!",
								 "author" : {
							   "givenName" : "John",
							   "familyName" : "Doe"
								 },
								 "tags":[ "example", "sample" ],
								 "content": "This will be unchanged"
						}
							)";
			std::string jpatch = R"(
						{
								 "title": "Hello!",
								 "phoneNumber": "+01-123-456-7890",
								 "author": {
							   "familyName": null
								 },
								 "tags": [ "example" ]
						}
							)";
			const char* ret = "";
			const char* exceptionText = "";
			int status = dllJE_jsoncons_merge_patch_apply(jtext.c_str(), jpatch.c_str(), &ret, &exceptionText);
			printf("merge_patch_apply return: %d [EXC: %s], %s\n", status, exceptionText, ret);
		}

		pfJE_jsoncons_merge_patch_create dllJE_jsoncons_merge_patch_create = (pfJE_jsoncons_merge_patch_create)GetProcAddress(hInstance, "JE_jsoncons_merge_patch_create");
		if (dllJE_jsoncons_merge_patch_create)
		{
			std::string source = R"(
			{
					 "title": "Goodbye!",
					 "author" : {
				   "givenName" : "John",
				   "familyName" : "Doe"
					 },
					 "tags":[ "example", "sample" ],
					 "content": "This will be unchanged"
			}
				)";
			std::string target = R"(
			{
			  "title": "Hello!",
			  "author": {
				"givenName": "John"
			  },
			  "tags": [
				"example"
			  ],
			  "content": "This will be unchanged",
			  "phoneNumber": "\u002B01-123-456-7890"
			}
				)";
			const char* ret = "";
			const char* exceptionText = "";
			int status = dllJE_jsoncons_merge_patch_create(source.c_str(), target.c_str(), &ret, &exceptionText);
			printf("merge_patch_create return: %d [EXC: %s], %s\n", status, exceptionText, ret);
		}
	}

	{
		pfJE_jsoncons_patch_apply dllJE_jsoncons_patch_apply = (pfJE_jsoncons_patch_apply)GetProcAddress(hInstance, "JE_jsoncons_patch_apply");
		if (dllJE_jsoncons_patch_apply)
		{
			std::string jtext = R"(
						{ "foo": "bar"}
							)";
			std::string jpatch = R"(
						[
							{ "op": "add", "path": "/baz", "value": "qux" },
							{ "op": "add", "path": "/foo", "value": [ "bar", "baz" ] }
						]
							)";
			const char* ret = "";
			const char* exceptionText = "";
			int status = dllJE_jsoncons_patch_apply(jtext.c_str(), jpatch.c_str(), &ret, &exceptionText);
			printf("patch_apply return: %d [EXC: %s], %s\n", status, exceptionText, ret);
		}

		pfJE_jsoncons_patch_create dllJE_jsoncons_patch_create = (pfJE_jsoncons_patch_create)GetProcAddress(hInstance, "JE_jsoncons_patch_create");
		if (dllJE_jsoncons_patch_create)
		{
			std::string source = R"(
			{
					 "title": "Goodbye!",
					 "author" : {
				   "givenName" : "John",
				   "familyName" : "Doe"
					 },
					 "tags":[ "example", "sample" ],
					 "content": "This will be unchanged"
			}
				)";
			std::string target = R"(
			{
			  "title": "Hello!",
			  "author": {
				"givenName": "John"
			  },
			  "tags": [
				"example"
			  ],
			  "content": "This will be unchanged",
			  "phoneNumber": "\u002B01-123-456-7890"
			}
				)";
			const char* ret = "";
			const char* exceptionText = "";
			int status = dllJE_jsoncons_patch_create(source.c_str(), target.c_str(), &ret, &exceptionText);
			printf("patch_create return: %d [EXC: %s], %s\n", status, exceptionText, ret);
		}
	}


	FreeLibrary(hInstance);
}
